@@ -41,20 +41,13 @@
 #' approach where also the uniqueness in sensible variables for each pattern
 #' spanned by the key variables are evaluated.
 #'
-#' @name measure_risk
-#' @aliases measure_risk-methods measure_risk,data.frame-method
-#' measure_risk,matrix-method measure_risk,sdcMicroObj-method measure_risk
-#' ldiversity ldiversity-methods ldiversity,data.frame-method
-#' ldiversity,matrix-method ldiversity,sdcMicroObj-method print.measure_risk
-#' print.ldiversity
-#' @docType methods
+#' @rdname measure_risk
 #' @param obj Object of class \code{\link{sdcMicroObj-class}}
 #' @param x Output of measure_risk() or ldiversity()
-
 #' @param ... see arguments below
 #' \itemize{
-#' \item{data}{Input data, either a matrix or a data.frame.}
-#' \item{keyVars}{Names of categorical key variables}
+#' \item{data}{Input data, a data.frame.}
+#' \item{keyVars}{names (or indices) of categorical key variables (for data-frame method)}
 #' \item{w}{name of variable containing sample weights}
 #' \item{hid}{name of the clustering variable, e.g. the household ID}
 #' \item{max_global_risk}{Maximal global risk for threshold computation}
@@ -62,23 +55,18 @@
 #' }
 #' @return A modified \code{\link{sdcMicroObj-class}} object or a list with the following elements:
 #' \itemize{
-#' \item{global_risk_ER}{expected number of re-identification.}
-#' \item{global_risk}{global risk (sum of indivdual risks).}
-#' \item{global_risk_pct}{global risk in percent.}
-#' \item{Res}{matrix with the risk, frequency in the sample and grossed-up frequency in the population (and the hierachical risk) for each observation.}
-#' \item{global_threshold}{for a given max_global_risk the threshold for the risk of observations.}
-#' \item{max_global_risk}{the input max_global_risk of the function.}
-#' \item{hier_risk_ER}{expected number of re-identification with household structure.}
-#' \item{hier_risk}{global risk with household structure (sum of indivdual risks).}
-#' \item{hier_risk_pct}{global risk with household structure in percent.}
-#' \item{ldiverstiy}{Matrix with Distinct_Ldiversity,
+#' \item{global_risk_ER: }{expected number of re-identification.}
+#' \item{global_risk: }{global risk (sum of indivdual risks).}
+#' \item{global_risk_pct: }{global risk in percent.}
+#' \item{Res: }{matrix with the risk, frequency in the sample and grossed-up frequency in the population (and the hierachical risk) for each observation.}
+#' \item{global_threshold: }{for a given max_global_risk the threshold for the risk of observations.}
+#' \item{max_global_risk: }{the input max_global_risk of the function.}
+#' \item{hier_risk_ER: }{expected number of re-identification with household structure.}
+#' \item{hier_risk: }{global risk with household structure (sum of indivdual risks).}
+#' \item{hier_risk_pct: }{global risk with household structure in percent.}
+#' \item{ldiverstiy: }{Matrix with Distinct_Ldiversity,
 #' Entropy_Ldiversity and Recursive_Ldiversity for each sensitivity variable.}}
-#' @section Methods: \describe{
-#' \item{list("signature(obj = \"data.frame\")")}{Method for object of class \dQuote{data.frame}}
-#' \item{list("signature(obj = \"matrix\")")}{Method for object of class \dQuote{matrix}}
-#' \item{list("signature(obj = \"sdcMicroObj\")")}{Method for object of S4 class \code{\link{sdcMicroObj-class}}}
-#' }
-#' @author Alexander Kowarik, Bernd Prantner, Matthias Templ, minor parts of IHSN C++ source
+#' @author Alexander Kowarik, Bernhard Meindl, Matthias Templ, Bernd Prantner, minor parts of IHSN C++ source
 #' @seealso \code{\link{freqCalc}}, \code{\link{indivRisk}}
 #' @references Franconi, L. and Polettini, S. (2004) \emph{Individual risk
 #' estimation in mu-Argus: a review}. Privacy in Statistical Databases, Lecture
@@ -92,7 +80,6 @@
 #' @keywords manip
 #' @export
 #' @examples
-#'
 #' ## measure_risk with sdcMicro objects:
 #' data(testdata)
 #' sdc <- createSdcObj(testdata,
@@ -138,29 +125,34 @@
 #'   numVars=c('expend','income','savings'), w='sampling_weight')
 #' ## already interally applied and availabe in object sdc:
 #' ## sdc <- measure_risk(sdc)
-#'
-setGeneric("measure_risk", function(obj, ...) {
-  standardGeneric("measure_risk")
+measure_risk <- function(obj, ...) {
+  measure_riskX(obj=obj, ...)
+}
+
+setGeneric("measure_riskX", function(obj, ...) {
+  standardGeneric("measure_riskX")
 })
 
-setMethod(f = "measure_risk", signature = c("sdcMicroObj"),
-  definition = function(obj, ...) {
-  origData <- get.sdcMicroObj(obj, type = "origData")
+setMethod(f="measure_riskX", signature=c("sdcMicroObj"),
+  definition=function(obj, ...) {
+  origData <- get.sdcMicroObj(obj, type="origData")
 
-  manipData <- get.sdcMicroObj(obj, type = "manipKeyVars")
+  manipData <- get.sdcMicroObj(obj, type="manipKeyVars")
   keyVars <- c(1:length(manipData))
-  w <- get.sdcMicroObj(obj, type = "weightVar")
+  w <- get.sdcMicroObj(obj, type="weightVar")
   if (length(w) > 0) {
     manipData <- cbind(manipData, origData[, w])
     w <- length(manipData)
   } else w <- NULL
-  hhId <- get.sdcMicroObj(obj, type = "hhId")
+  hhId <- get.sdcMicroObj(obj, type="hhId")
   if (length(hhId) > 0) {
     manipData <- cbind(manipData, origData[, hhId])
     hhId <- length(manipData)
   } else hhId <- NULL
-  res <- measure_riskWORK(manipData, keyVars, w = w, hid = hhId, ...)
-  risk <- get.sdcMicroObj(obj, type = "risk")
+
+  alpha <- get.sdcMicroObj(obj, type="options")$alpha
+  res <- measure_riskWORK(manipData, keyVars, w=w, hid=hhId, alpha=alpha, ...)
+  risk <- get.sdcMicroObj(obj, type="risk")
   risk$global <- list()
   risk$global$risk <- res$global_risk
   risk$global$risk_ER <- res$global_risk_ER
@@ -174,21 +166,22 @@ setMethod(f = "measure_risk", signature = c("sdcMicroObj"),
   }
   risk$individual <- res$Res
 
-  obj <- set.sdcMicroObj(obj, type = "risk", input = list(risk))
+  obj <- set.sdcMicroObj(obj, type="risk", input=list(risk))
   obj
 })
 
-setMethod(f = "measure_risk", signature = c("data.frame"),
-definition = function(obj, ...) {
-  measure_riskWORK(data = obj, ...)
+setMethod(f="measure_riskX", signature=c("data.frame"),
+definition=function(obj, ...) {
+  params <- list(...)
+  if (!is.null(params$alpha)) {
+    alpha <- params$alpha
+  } else {
+    alpha <- 1
+  }
+  measure_riskWORK(data=obj, alpha=alpha, ...)
 })
 
-setMethod(f = "measure_risk", signature = c("matrix"),
-definition = function(obj, ...) {
-  measure_riskWORK(data = obj, ...)
-})
-
-measure_riskWORK <- function(data, keyVars, w = NULL, missing = -999, hid = NULL, max_global_risk = 0.01, fast_hier = TRUE) {
+measure_riskWORK <- function(data, keyVars, w=NULL, missing=-999, hid=NULL, max_global_risk=0.01, fast_hier=TRUE, alpha) {
   if (!is.data.frame(data)) {
     data <- as.data.frame(data)
   }
@@ -207,16 +200,16 @@ measure_riskWORK <- function(data, keyVars, w = NULL, missing = -999, hid = NULL
         weight_variable <- colnames(data)[weight_variable] else stop("Weight variable not found!")
     }
   }
-  TFcharacter <- lapply(data[, keyVars, drop = FALSE], class) %in% "character"
+  TFcharacter <- lapply(data[, keyVars, drop=FALSE], class) %in% "character"
   if (any(TFcharacter)) {
     for (kvi in which(TFcharacter)) {
       data[, keyVars[kvi]] <- as.factor(data[, keyVars[kvi]])
     }
   }
 
-  f <- freqCalc(data, keyVars = keyVars, w = w)
-  ir <- indivRisk(f, survey = !is.null(w))
-  Res <- matrix(NA, ncol = 3, nrow = nrow(data))
+  f <- freqCalc(data, keyVars=keyVars, w=w, alpha=alpha)
+  ir <- indivRisk(f, survey=!is.null(w))
+  Res <- matrix(NA, ncol=3, nrow=nrow(data))
   Res[, 1] <- ir$rk
   Res[, 2] <- ir$fk
   Res[, 3] <- f$Fk
@@ -226,7 +219,7 @@ measure_riskWORK <- function(data, keyVars, w = NULL, missing = -999, hid = NULL
     weighted <- 1
   }
   n_key_vars <- length(variables)
-  dataX <- data[, c(variables), drop = FALSE]
+  dataX <- data[, c(variables), drop=FALSE]
   for (i in 1:ncol(dataX)) {
     if (!is.numeric(dataX[, i])) {
       dataX[, i] <- as.numeric(dataX[, i])
@@ -237,17 +230,17 @@ measure_riskWORK <- function(data, keyVars, w = NULL, missing = -999, hid = NULL
   }
   dataX <- as.matrix(dataX)
   ind <- do.call(order, data.frame(dataX))
-  dataX <- dataX[ind, , drop = FALSE]
+  dataX <- dataX[ind, , drop=FALSE]
   ind <- order(c(1:nrow(dataX))[ind])
   # res <- .Call('measure_risk',dataX,weighted,n_key_vars,2,-99,missing)#
   res <- list()
   res$Res <- Res
   colnames(res$Res) <- c("risk", "fk", "Fk")
-  res$global_risk_ER <- sum(ir$rk, na.rm = TRUE)
+  res$global_risk_ER <- sum(ir$rk, na.rm=TRUE)
   res$global_risk <- res$global_risk_ER/nrow(res$Res)
   res$global_risk_pct <- res$global_risk * 100
 
-  ind <- order(res$Res[, 1], decreasing = TRUE)
+  ind <- order(res$Res[, 1], decreasing=TRUE)
   if (max_global_risk >= 1 || max_global_risk <= 0) {
     stop("max_global_risk argument must be between 0 and 1!")
   }
@@ -268,20 +261,20 @@ measure_riskWORK <- function(data, keyVars, w = NULL, missing = -999, hid = NULL
         dataX[, i] <- as.numeric(dataX[, i])
     }
     dataX <- as.matrix(dataX)
-    maxHH <- max(table(dataX[, 1]), na.rm = TRUE)
+    maxHH <- max(table(dataX[, 1]), na.rm=TRUE)
     if (fast_hier) {
-      # warning('The households are to large for a fast computation of the hierachical risk.\n
+      # warning('The households are too large for a fast computation of the hierachical risk.\n
       # (Use the parameter forceHier to perform the computation anyway)')
-      reshier <- by(dataX[, 2], dataX[, 1], function(x) 1 - prod(1 - x), simplify = TRUE)
+      reshier <- by(dataX[, 2], dataX[, 1], function(x) 1 - prod(1 - x), simplify=TRUE)
       reshier <- data.frame(cbind(reshier, unique(dataX[, 1])))
-      colnames(reshier) = c("reshier", "hhid")
+      colnames(reshier)=c("reshier", "hhid")
       dataX <- data.frame(dataX[, 1])
-      colnames(dataX) = c("hhid")
-      datX <- merge(dataX, reshier, all.x = TRUE)
+      colnames(dataX)=c("hhid")
+      datX <- merge(dataX, reshier, all.x=TRUE)
 
       res$Res <- cbind(res$Res, datX$reshier[ind])
-      res[["hier_risk_ER"]] <- sum(res$Res[, 4], na.rm = TRUE)
-      res[["hier_risk"]] <- sum(res$Res[, 4], na.rm = TRUE)/nrow(res$Res)
+      res[["hier_risk_ER"]] <- sum(res$Res[, 4], na.rm=TRUE)
+      res[["hier_risk"]] <- sum(res$Res[, 4], na.rm=TRUE)/nrow(res$Res)
       res[["hier_risk_pct"]] <- res[["hier_risk"]] * 100
     } else {
       resh <- .Call("measure_hierachical", dataX)
@@ -301,14 +294,16 @@ measure_riskWORK <- function(data, keyVars, w = NULL, missing = -999, hid = NULL
 #' @param missing a integer value to be used as missing value in the C++ routine
 #' @param ldiv_index indices (or names) of the variables used for l-diversity
 #' @param l_recurs_c l-Diversity Constant
-#' @note internal function
-#' @author Bernhard Meindl \email{bernhard.meindl@@statistik.gv.at}
-setGeneric("ldiversity", function(obj, ldiv_index=NULL, l_recurs_c = 2, missing = -999, ...) {
-  standardGeneric("ldiversity")
+ldiversity <- function(obj, ldiv_index=NULL, l_recurs_c=2, missing=-999, ...) {
+  ldiversityX(obj=obj, ldiv_index=ldiv_index, l_recurs_c=l_recurs_c, missing=missing, ...)
+}
+
+setGeneric("ldiversityX", function(obj, ldiv_index=NULL, l_recurs_c=2, missing=-999, ...) {
+  standardGeneric("ldiversityX")
 })
 
-setMethod(f = "ldiversity", signature = c("sdcMicroObj"),
-definition = function(obj, ldiv_index=NULL, l_recurs_c = 2, missing = -999) {
+setMethod(f="ldiversityX", signature=c("sdcMicroObj"),
+definition=function(obj, ldiv_index=NULL, l_recurs_c=2, missing=-999) {
   o <- obj@origData
   k <- obj@manipKeyVars
   n <- obj@manipNumVars
@@ -332,21 +327,16 @@ definition = function(obj, ldiv_index=NULL, l_recurs_c = 2, missing = -999) {
   if (!is.null(s))
     o$sdcGUI_strataVar <- s
   kV <- colnames(obj@origData)[get.sdcMicroObj(obj, "keyVars")]
-  obj@risk$ldiversity <- ldiversityWORK(data = o, keyVars = kV, l_recurs_c = l_recurs_c, ldiv_index = ldiv_index, missing = missing)
+  obj@risk$ldiversity <- ldiversityWORK(data=o, keyVars=kV, l_recurs_c=l_recurs_c, ldiv_index=ldiv_index, missing=missing)
   return(obj)
 })
 
-setMethod(f = "ldiversity", signature = c("data.frame"),
-definition = function(obj, keyVars, ldiv_index, l_recurs_c = 2, missing = -999) {
-  ldiversityWORK(data = obj, keyVars = keyVars, ldiv_index = ldiv_index, l_recurs_c = l_recurs_c, missing = missing)
+setMethod(f="ldiversityX", signature=c("data.frame"),
+definition=function(obj, keyVars, ldiv_index, l_recurs_c=2, missing=-999) {
+  ldiversityWORK(data=obj, keyVars=keyVars, ldiv_index=ldiv_index, l_recurs_c=l_recurs_c, missing=missing)
 })
 
-setMethod(f = "ldiversity", signature = c("matrix"), definition = function(obj, keyVars, ldiv_index,
-  l_recurs_c = 2, missing = -999) {
-  ldiversityWORK(data = obj, keyVars = keyVars, ldiv_index = ldiv_index, l_recurs_c = l_recurs_c, missing = missing)
-})
-
-ldiversityWORK <- function(data, keyVars, ldiv_index, missing = -999, l_recurs_c = 2) {
+ldiversityWORK <- function(data, keyVars, ldiv_index, missing=-999, l_recurs_c=2) {
   variables <- keyVars
   if ((is.null(variables) || !variables %in% colnames(data)) && is.character(variables))
     stop("Please define valid key variables") else if (is.numeric(variables)) {
@@ -366,17 +356,17 @@ ldiversityWORK <- function(data, keyVars, ldiv_index, missing = -999, l_recurs_c
   } else ldiv_var <- character(0)
 
   n_key_vars <- length(variables)
-  dataX <- data[, c(variables, ldiv_var), drop = FALSE]
+  dataX <- data[, c(variables, ldiv_var), drop=FALSE]
   for (i in 1:ncol(dataX)) {
     if (!is.numeric(dataX[, i]))
       dataX[, i] <- as.numeric(dataX[, i])
   }
   dataX <- as.matrix(dataX)
   ind <- do.call(order, data.frame(dataX))
-  dataX <- dataX[ind, , drop = FALSE]
+  dataX <- dataX[ind, , drop=FALSE]
   ind <- order(c(1:nrow(dataX))[ind])
   if (is.null(ldiv_index))
-    ldiv_index = -99
+    ldiv_index=-99
   if (length(ldiv_index) > 5)
     stop("Maximal number of sensitivity variables is 5")
   res <- .Call("measure_risk", dataX, 0, n_key_vars, l_recurs_c, ldiv_index, missing)
@@ -385,25 +375,24 @@ ldiversityWORK <- function(data, keyVars, ldiv_index, missing = -999, l_recurs_c
   if (all(ldiv_index != -99)) {
     res$Mat_Risk <- res$Mat_Risk[ind, ]
     names(res)[names(res) == "Mat_Risk"] <- "ldiversity"
-    colnames(res$ldiversity) <- c(paste(rep(ldiv_var, each = 3), rep(c("Distinct_Ldiversity",
-      "Entropy_Ldiversity", "Recursive_Ldiversity"), length(ldiv_index)), sep = "_"),
+    colnames(res$ldiversity) <- c(paste(rep(ldiv_var, each=3), rep(c("Distinct_Ldiversity",
+      "Entropy_Ldiversity", "Recursive_Ldiversity"), length(ldiv_index)), sep="_"),
       "MultiEntropy_Ldiversity", "MultiRecursive_Ldiversity")
   } else {
     res <- res[names(res) != "Mat_Risk"]
   }
-  ind <- order(res$Res[, 1], decreasing = TRUE)
+  ind <- order(res$Res[, 1], decreasing=TRUE)
   res <- res$ldiversity
   class(res) <- "ldiversity"
   invisible(res)
 }
 
-#' Print method for objects from class ldiversity
+#' Print method for objects of class measure_risk
+#'
+#' Prints a 'measure_risk'-object
 #'
-#' Print method for objects from class ldiversity
 #' @rdname measure_risk
 #' @return Prints risk-information into the console
-#' @author Bernhard Meindl, Matthias Templ
-#' @seealso \code{\link{measure_risk}}
 #' @keywords print
 #' @method print measure_risk
 #' @export
@@ -436,10 +425,10 @@ print.measure_risk <- function(x, ...) {
 
 #' Print method for objects from class ldiversity
 #'
-#' Print method for objects from class ldiversity
+#' Prints a 'ldiversity'-object
+#'
 #' @rdname measure_risk
 #' @return Information on L-Diversity Measures in the console
-#' @author Bernhard Meindl, Matthias Templ
 #' @seealso \code{\link{measure_risk}}
 #' @keywords print
 #' @method print ldiversity
